
% CWC paper - revision 2017-01-02

clear all
clc
global B_backup; % this one is for storing B in case iterations are interrupted with ctrl-c


%% ****************************  loading data  ****************************


EstimOpt.fnameinput = ('data_CWC_04');
DATA = load(EstimOpt.fnameinput);


%% ****************************  data transformations  ****************************


DATA.SQ = zeros(length(DATA.alt),1);
DATA.SQ(DATA.alt==3,1) = 1;
DATA.size5 = zeros(length(DATA.alt),1);
DATA.size5(DATA.size==5) = 1;
DATA.size10 = zeros(length(DATA.alt),1);
DATA.size10(DATA.size==10) = 1;


%% ****************************  model specification  ****************************


DATA.Y = DATA.ANS;

DATA.Xa = [DATA.SQ, DATA.size, DATA.comm.*DATA.size5, DATA.comm.*DATA.size10, DATA.fish.*DATA.size5, DATA.fish.*DATA.size10, DATA.hab.*DATA.size5,DATA.hab.*DATA.size10, -DATA.cost*1000*0.115/100];
EstimOpt.NamesA = {'SQ';'size (1000 km2)'; 'oil/gas*size5'; 'oil/gas*size10'; 'fishing*size5'; 'fishing*size10'; 'habitat*size5'; 'habitat*size10'; '-cost * scale'}; % Specyfy names of the attributes


%% ****************************  specifying input ****************************


DATA.filter = ones(size(DATA.Y))==1;

INPUT.Y = DATA.Y(DATA.filter);
INPUT.Xa = DATA.Xa(DATA.filter,:);

INPUT.MissingInd = DATA.out(DATA.filter);


%% ****************************  sample characteristics ****************************


EstimOpt.NCT = 12; % Number of choice tasks per person 
EstimOpt.NAlt = 3; % Number of alternatives
EstimOpt.NP = length(INPUT.Y)/EstimOpt.NCT/EstimOpt.NAlt; % 397


%% **************************** estimation and optimization options ****************************

% EstimOpt.eps = 1.e-6; % overall precision level

[INPUT, Results, EstimOpt, OptimOpt] = DataCleanDCE(INPUT,EstimOpt);

EstimOpt.NRep = 1e4; % number of draws for numerical simulation
% OptimOpt.MaxIter = 1e1; % maximum number of iterations

% OptimOpt.Algorithm = 'trust-region'; % 'quasi-newton';
% EstimOpt.NumGrad = 1; % 1
% OptimOpt.GradObj = 'off'; % 'off'
% OptimOpt.FinDiffType = 'central'; % 'forward'
% OptimOpt.Hessian = 'user-supplied'; % 'off'
% EstimOpt.HessEstFix = 1; % 0 = use optimization Hessian, 1 = use jacobian-based (BHHH) Hessian, 2 - use high-precision jacobian-based (BHHH) Hessian 3 - use numerical Hessian
% EstimOpt.ApproxHess = 0;


%% ****************************     MNL     ****************************


EstimOpt.WTP_space = 1; % number of cost parameters for WTP-space estimation (need to come last in Xa)

Results.MNL = MNL(INPUT,Results,EstimOpt,OptimOpt);


%% ****************************     MXL_d     ****************************


% EstimOpt.FullCov = 0; % Specifing if random parametrs are correlated or not (default)

EstimOpt.Dist = [1;zeros(size(INPUT.Xa,2)-1,1);1]; % 0 - normal, -1 - constant, 1 - lognormal, 2 - Spike, 5 - Weibull; the first one is for the scale parameter (include even for MXL) 

% -3626.3146
Results.MXL_d.b0 = [-1.41910236214087;-0.0484501658437971;-0.157399770704711;0.234922387303157;0.0400936004412222;0.336030383147162;1.42521308690329;1.63608255192576;0.0685078933040307;3.69629199013202;-0.104445606359171;1.05220901017126;1.39492249959827;0.959862635618499;1.38759151204933;1.56597140554000;1.57705861025535;0.506412736169641];

Results.MXL_d = MXL(INPUT,Results,EstimOpt,OptimOpt);


%% ****************************     MXL     ****************************

EstimOpt.FullCov = 1; % Specify if random parametrs are correlated or not (default)

% -3438.5176
Results.MXL.b0 = [-2.05590746953430;-0.232078773515390;-0.497566469491630;0.712337149609879;0.0823424114279229;0.619537177100191;1.36731518338312;1.84707476683611;0.458245547021195;3.22583567354848;0.0155929783009290;0.451886836923307;-0.964127629843721;0.130125486862157;-0.527138289733984;0.173682031749371;0.373747677462804;0.431743055750043;0.261084649328827;-0.363923384520579;-0.369958080859154;-0.233473320962889;-0.652331344347881;0.126304980803397;-0.690798462576842;0.0982790614592804;0.446312453475324;0.451927929228710;0.144899716449236;-0.294248264230550;0.0706979230514499;-0.106821336218465;0.0911348111151292;1.28118162861759;0.0443390572086996;0.272757850889286;0.633679987622188;0.167173057764146;-0.139132430035533;0.651653329332686;1.08279289696882;0.00335297893017951;-0.0539684898452671;-0.316989777201829;0.363456172381392;0.621719191448655;0.809336023126810;0.0402010453200035;1.02160073997165;1.28445891676296;-0.271483995224407;-0.352551616519050;-0.388309697380225;0.303374064451873];

Results.MXL = MXL(INPUT,Results,EstimOpt,OptimOpt);



%% simulate WTP:

B = Results.MXL.bhat;
V = Results.MXL.ihess;

% This can be time consuming - change both to e.g. 1e4 for less precise but faster results
sim1 = 1e5;
sim2 = 1e5;

NVarA = EstimOpt.NVarA;

bhat = mvnrnd(B,V,sim1)';
WTP_i = zeros(sim1,2*5);

parfor i = 1:sim1
    bhat_i = bhat(:,i);
%     B0_i = mvnrnd(bhat_i(1:NVarA)',diag(bhat_i(NVarA+1:NVarA*2).^2),sim2);
    b0v = bhat_i(NVarA+1:end)';
    VC = tril(ones(NVarA));
    VC(VC==1) = b0v;
    VC = VC*VC';
    B0_i = mvnrnd(bhat_i(1:NVarA)',VC,sim2);
    WTP_tmp = [-B0_i(:,1) + -B0_i(:,2)*(5-2.445) + B0_i(:,3) + B0_i(:,5) + B0_i(:,7),-B0_i(:,1) + -B0_i(:,2)*(10-2.445) + B0_i(:,4) + B0_i(:,6) + B0_i(:,8)];
    WTP_i(i,:) = [mean(WTP_tmp) median(WTP_tmp) std(WTP_tmp) quantile(WTP_tmp,0.025), quantile(WTP_tmp,0.975)];
end

Wm = median(WTP_i);
Ws = std(WTP_i);
Wq1 = quantile(WTP_i,0.025);
Wq2 = quantile(WTP_i,0.975);

RES = reshape(permute(reshape([Wm; Ws; Wq1; Wq2],[4,2,5]),[1,3,2]),[4*5,2]);
